@@ -1,7 +1,7 @@
 package Bio::Tools::Prepeat;
 use 5.006;
 use strict;
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 use XSLoader;
 XSLoader::load 'Bio::Tools::Prepeat';
 use Exporter;
@@ -11,7 +11,7 @@ use Cwd qw(abs_path);
 use String::Random qw(random_string);
 use IO::File;
 
-our @amino = qw/A C D E F G H I L M N P Q R S T V W Y Z/;
+our @amino = 'A'..'Z';
 sub random_sequence { ref($_[0]) ? shift : undef;random_string('0'x(shift), \@amino) }
 
 sub new {
@@ -63,7 +63,6 @@ sub acclen {
 	$sum += length $pkg->{seqarr}->[$_-1];
 	push @{$pkg->{acclen}}, $sum;
     }
-
 }
 
 sub loadidx {
@@ -107,7 +106,7 @@ sub relpos {
     my ($pkg, $abspos) = @_;
     my ($seqid, $pos) = (0, 0);
     for( my $i=$#{$pkg->{acclen}} ; $i>=0 ; $i--){
-	if( $abspos > $pkg->{acclen}->[$i] ){
+	if( $abspos >= $pkg->{acclen}->[$i] ){
 	    $pos = $abspos - $pkg->{acclen}->[$i];
 	    $seqid = $i;
 	    last;
@@ -120,9 +119,15 @@ sub coincidence_length {
     my ($pkg, $prev, $this) = @_;
     my @prevrel = $pkg->relpos($prev);
     my @thisrel = $pkg->relpos($this);
-    return undef unless
-	substr($pkg->{seqarr}->[$prevrel[0]], $prevrel[1], $pkg->{length})
-	    eq substr($pkg->{seqarr}->[$thisrel[0]], $thisrel[1], $pkg->{length});
+    my $str_a = substr($pkg->{seqarr}->[$prevrel[0]], $prevrel[1], $pkg->{length});
+    my $str_b = substr($pkg->{seqarr}->[$thisrel[0]], $thisrel[1], $pkg->{length});
+
+    return undef if length $str_a != $pkg->{length} && $str_b != $pkg->{length};
+    return undef if $str_a ne $str_b;
+#    print "$prev, $this => ", substr($pkg->{seqarr}->[$prevrel[0]], $prevrel[1], $pkg->{length}), '/', substr($pkg->{seqarr}->[$thisrel[0]], $thisrel[1], $pkg->{length}), $/;
+
+
+
     return [ \@prevrel, \@thisrel ];
 }
 
@@ -134,22 +139,27 @@ sub query {
     open R, '>', $pkg->{wd}."/result";
     my ($prev, $this);
     foreach (bigram_set){
+	next unless -s $pkg->{wd}."/idx.$_";
 	open F, $pkg->{wd}."/idx.$_" or die "Cannot open index file $_ for query\n";
 	my @posarr;
-	while ( chomp ($_ = <F>) ){ push @posarr, $_ }
+	while ( chomp ($_ = <F>) ){ push @posarr, $_; }
+
+	my %checked;
 	foreach $prev (@posarr){
+	    next if $checked{$prev};
 	    foreach $this (@posarr){
-		last if $this < $prev;
 		if($this - $prev > $pkg->{length}){
 		    my $occ = $pkg->coincidence_length($prev, $this);
 		    if(ref $occ){
+			$checked{$this} = 1;
 			print R
 			    join ' ',
 			    substr($pkg->{seqarr}->[$occ->[0]->[0]],
 				   $occ->[0]->[1], $pkg->{length}),
 			    "@{$occ->[0]} @{$occ->[1]}\n";
+
 		    }
-		} 
+		}
 	    }
 	}
 	close F;
@@ -161,7 +171,9 @@ sub query {
     my $ret;
     while(chomp($_=<R>)){
 	my @e = split /\s/, $_;
-	push @{$ret->{$e[0]}}, $e[0] != $prep ? ([ @e[1..2] ], [ @e[3..4] ]) : [ @e[3..4] ];
+	next unless length $e[0]  == $length;
+	push @{$ret->{$e[0]}}, ($e[0] ne $prep ? ([ @e[1..2] ], [ @e[3..4] ]) : [ @e[3..4] ]);
+	$prep = $e[0];
     }
     close R;
     $ret;
